var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from "react";
import PropTypes from "prop-types";

import { Portal } from "../internal/Portal";
import { MapContext } from "../internal/MapContext";

/**
 * Controls display options of custom control.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, CustomControl } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <CustomControl>
 *         <button onClick={props.onControlClick}>
 *           Click Me
 *         </button>
 *       </CustomControl>
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [Custom Controls](https://developers.google.com/maps/documentation/javascript/examples/control-custom)
 */
export var CustomControl = function (_React$Component) {
  _inherits(CustomControl, _React$Component);

  function CustomControl() {
    _classCallCheck(this, CustomControl);

    return _possibleConstructorReturn(this, (CustomControl.__proto__ || Object.getPrototypeOf(CustomControl)).apply(this, arguments));
  }

  _createClass(CustomControl, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      this.div = document.createElement("div");

      this.mountControl();
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps) {
      var _props = this.props,
          position = _props.position,
          children = _props.children;


      return position !== nextProps.position || children !== nextProps.children;
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      if (this.props.position !== nextProps.position) {
        // We need to remove control from old position first.
        this.unmountControl();

        // And only after that we can add control to new position.
        this.mountControl(nextProps);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.unmountControl();
    }
  }, {
    key: "getControls",
    value: function getControls() {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props,
          position = _ref.position;

      var mapContext = this.context.mapContext;


      return mapContext.map.controls[mapContext.getEnum("ControlPosition", position)];
    }
  }, {
    key: "mountControl",
    value: function mountControl(props) {
      var controls = this.getControls(props);

      if (controls) {
        var index = controls.indexOf(this.div);

        if (index === -1) {
          controls.push(this.div);
        }
      }
    }
  }, {
    key: "unmountControl",
    value: function unmountControl(props) {
      var controls = this.getControls(props);

      if (controls) {
        var index = controls.indexOf(this.div);

        if (index !== -1) {
          controls.removeAt(index);
        }
      }
    }
  }, {
    key: "render",
    value: function render() {
      var controls = this.getControls();

      return !controls ? null : React.createElement(
        Portal,
        { node: this.div },
        this.props.children
      );
    }
  }]);

  return CustomControl;
}(React.Component);

CustomControl.contextTypes = {
  mapContext: PropTypes.instanceOf(MapContext).isRequired
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  CustomControl.propTypes = {
    /**
     * Content of control.
     */
    children: PropTypes.element.isRequired,

    /**
     * Position id.
     *
     * Used to specify the position of the control on the map.
     *
     * See also: [google.maps.ControlPosition](https://developers.google.com/maps/documentation/javascript/reference#ControlPosition)
     */
    position: PropTypes.oneOf(["BOTTOM_CENTER", "BOTTOM_LEFT", "BOTTOM_RIGHT", "LEFT_BOTTOM", "LEFT_CENTER", "LEFT_TOP", "RIGHT_BOTTOM", "RIGHT_CENTER", "RIGHT_TOP", "TOP_CENTER", "TOP_LEFT", "TOP_RIGHT"]).isRequired
  };
}