import _map from "lodash-es/map";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from "react";

import PropTypes from "prop-types";
import DrawingControlEvents from "./DrawingControlEvents";
import { isEqualProps, createListeners } from "../internal/Utils";
import { MapContext } from "../internal/MapContext";

/**
 * Controls display options of `google.maps.drawing.DrawingManager` control.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, DrawingControl } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <DrawingControl onOverlayComplete={props.onOverlayComplete} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.drawing.DrawingManager](https://developers.google.com/maps/documentation/javascript/3.exp/reference#DrawingManager)
 * * [google.maps.drawing.DrawingManagerOptions](https://developers.google.com/maps/documentation/javascript/3.exp/reference#DrawingManagerOptions)
 */
export var DrawingControl = function (_React$Component) {
  _inherits(DrawingControl, _React$Component);

  function DrawingControl() {
    _classCallCheck(this, DrawingControl);

    return _possibleConstructorReturn(this, (DrawingControl.__proto__ || Object.getPrototypeOf(DrawingControl)).apply(this, arguments));
  }

  _createClass(DrawingControl, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var mapContext = this.context.mapContext;


      var options = this.getOptions();
      var drawingManager = new mapContext.maps.drawing.DrawingManager();

      this.drawingManager = drawingManager;

      drawingManager.setValues(options);
      drawingManager.setMap(mapContext.map);

      drawingManager.addListener(DrawingControlEvents.onOverlayComplete, function (x) {
        x.overlay.setMap(null);
      });

      createListeners(DrawingControlEvents, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        drawingManager.addListener(event, listener);
      });
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var prevOptions = this.getOptions();
      var nextOptions = this.getOptions(nextProps);

      if (!isEqualProps(prevOptions, nextOptions)) {
        this.drawingManager.setValues(nextOptions);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.drawingManager.setMap(null);
    }
  }, {
    key: "getOptions",
    value: function getOptions() {
      var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props,
          position = _ref3.position,
          drawingModes = _ref3.drawingModes;

      var mapContext = this.context.mapContext;


      return {
        drawingControl: true,
        drawingControlOptions: {
          position: mapContext.getEnum("ControlPosition", position),
          drawingModes: _map(drawingModes, function (x) {
            return mapContext.getEnum("drawing.OverlayType", x);
          })
        }
      };
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return DrawingControl;
}(React.Component);

DrawingControl.contextTypes = {
  mapContext: PropTypes.instanceOf(MapContext).isRequired
};

DrawingControl.defaultProps = {
  position: "TOP_LEFT",
  drawingModes: ["circle", "marker", "polygon", "polyline", "rectangle"]
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  DrawingControl.propTypes = {
    /**
     * Position id.
     *
     * Used to specify the position of the control on the map.
     *
     * See also: [google.maps.ControlPosition](https://developers.google.com/maps/documentation/javascript/reference#ControlPosition)
     */
    position: PropTypes.oneOf(["BOTTOM_CENTER", "BOTTOM_LEFT", "BOTTOM_RIGHT", "LEFT_BOTTOM", "LEFT_CENTER", "LEFT_TOP", "RIGHT_BOTTOM", "RIGHT_CENTER", "RIGHT_TOP", "TOP_CENTER", "TOP_LEFT", "TOP_RIGHT"]),

    /**
     * The drawing modes to display in the drawing control, in the order in
     * which they are to be displayed.
     *
     * The hand icon (which corresponds to the null drawing mode)
     * is always available and is not to be specified in this array.
     */
    drawingModes: PropTypes.arrayOf(PropTypes.oneOf(["circle", "marker", "polygon", "polyline", "rectangle"])),

    /**
     * This handler is called when the user has finished drawing a `circle`.
     */
    onCircleComplete: PropTypes.func,

    /**
     * This handler is called when the user has finished drawing a `marker`.
     */
    onMarkerComplete: PropTypes.func,

    /**
     * This handler is called when the user has finished drawing an `overlay`
     * of any type.
     */
    onOverlayComplete: PropTypes.func,

    /**
     * This handler is called when the user has finished drawing a `polygon`.
     */
    onPolygonComplete: PropTypes.func,

    /**
     * This handler is called when the user has finished drawing a `polyline`.
     */
    onPolylineComplete: PropTypes.func,

    /**
     * This handler is called when the user has finished drawing a `rectangle`.
     */
    onRectangleComplete: PropTypes.func
  };
}