import _pick from "lodash-es/pick";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from "react";

import PropTypes from "prop-types";
import InfoWindowEvents from "./InfoWindowEvents";
import { isEqualProps, createListeners } from "../internal/Utils";
import { Portal } from "../internal/Portal";
import { MapContext } from "../internal/MapContext";

var pickProps = function pickProps(x) {
  return _pick(x, ["position", "maxWidth", "zIndex", "pixelOffset", "disableAutoPan"]);
};

/**
 * Draws `google.maps.InfoWindow`.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, InfoWindow } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <InfoWindow position={props.center}>
 *         <div>Hello World!</div>
 *       </InfoWindow>
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.InfoWindow](https://developers.google.com/maps/documentation/javascript/reference#InfoWindow)
 * * [google.maps.InfoWindowOptions](https://developers.google.com/maps/documentation/javascript/reference#InfoWindowOptions)
 */
export var InfoWindow = function (_React$Component) {
  _inherits(InfoWindow, _React$Component);

  function InfoWindow() {
    _classCallCheck(this, InfoWindow);

    return _possibleConstructorReturn(this, (InfoWindow.__proto__ || Object.getPrototypeOf(InfoWindow)).apply(this, arguments));
  }

  _createClass(InfoWindow, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var mapContext = this.context.mapContext;


      var options = this.getOptions();
      var infoWindow = new mapContext.maps.InfoWindow();

      infoWindow.setValues(options);

      this.infoWindow = infoWindow;
      this.div = document.createElement("div");

      this.updateContent();
      this.updateVisibility();

      infoWindow.addListener(InfoWindowEvents.onCloseClick, function () {
        if (_this2.props.open) {
          infoWindow.open(mapContext.map);
        }
      });

      createListeners(InfoWindowEvents, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        infoWindow.addListener(event, listener);
      });
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var _props = this.props,
          open = _props.open,
          children = _props.children,
          maxWidth = _props.maxWidth;


      var prevOptions = this.getOptions();
      var nextOptions = this.getOptions(nextProps);

      if (!isEqualProps(prevOptions, nextOptions)) {
        this.infoWindow.setValues(nextOptions);
      }

      if (nextProps.children !== children) {
        this.updateContent(nextProps);
      }

      if (Boolean(nextProps.open) !== Boolean(open) || Boolean(nextProps.maxWidth !== maxWidth && nextProps.open)) {
        this.updateVisibility(nextProps);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.infoWindow.close();
      this.context.mapContext.maps.event.clearInstanceListeners(this.infoWindow);
    }
  }, {
    key: "getOptions",
    value: function getOptions() {
      var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props;

      var options = pickProps(props);

      if (options.pixelOffset) {
        options.pixelOffset = this.context.mapContext.createSize(options.pixelOffset);
      }

      return options;
    }
  }, {
    key: "updateContent",
    value: function updateContent() {
      var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props,
          children = _ref3.children;

      this.infoWindow.setContent(React.isValidElement(children) ? this.div : children);
    }
  }, {
    key: "updateVisibility",
    value: function updateVisibility() {
      var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props,
          open = _ref4.open;

      if (open) {
        this.infoWindow.open(this.context.mapContext.map);
      } else {
        this.infoWindow.close();
      }
    }
  }, {
    key: "render",
    value: function render() {
      var _props2 = this.props,
          open = _props2.open,
          children = _props2.children;

      return !open || !React.isValidElement(children) ? null : React.createElement(
        Portal,
        { node: this.div },
        children
      );
    }
  }]);

  return InfoWindow;
}(React.Component);

InfoWindow.defaultProps = {
  open: true,
  disableAutoPan: false
};

InfoWindow.contextTypes = {
  mapContext: PropTypes.instanceOf(MapContext).isRequired
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  InfoWindow.propTypes = {
    /**
     * 	Controls whether the `InfoWindow` is opened or not.
     */
    open: PropTypes.bool,

    /**
     * This handler is called when the close button was clicked.
     */
    onCloseClick: PropTypes.func,

    /**
     * The LatLng at which to display this `InfoWindow`.
     */
    position: PropTypes.shape({
      /**
       * The latitude in degrees.
       */
      lat: PropTypes.number.isRequired,
      /**
       * The longitude in degrees.
       */
      lng: PropTypes.number.isRequired
    }).isRequired,

    /**
     * Content to display in the `InfoWindow`.
     */
    children: PropTypes.node.isRequired,

    /**
     * Disable auto-pan on open.
     */
    disableAutoPan: PropTypes.bool,

    /**
     * Maximum width of the `InfoWindow`, regardless of content's width.
     */
    maxWidth: PropTypes.number,

    /**
     * The offset, in pixels, of the tip of the info window from the point
     * on the map at whose geographical coordinates the info window is anchored.
     */
    pixelOffset: PropTypes.shape({
      /**
       * The height along the y-axis, in pixels.
       */
      width: PropTypes.number.isRequired,
      /**
       * The width along the x-axis, in pixels.
       */
      height: PropTypes.number.isRequired
    }),

    /**
     * All InfoWindows are displayed on the map in order of their zIndex,
     * with higher values displaying in front of InfoWindows with lower values.
     *
     * By default, InfoWindows are displayed according to their latitude,
     * with InfoWindows of lower latitudes appearing in front of InfoWindows
     * at higher latitudes.
     *
     * InfoWindows are always displayed in front of markers.
     */
    zIndex: PropTypes.number
  };
}