import _union from "lodash-es/union";
import _keys from "lodash-es/keys";
import _isEqualWith from "lodash-es/isEqualWith";
import _isFunction from "lodash-es/isFunction";
import React from "react";


// eslint-disable-next-line consistent-return
var isEqualPropsCustomizer = function isEqualPropsCustomizer(a, b) {
  // Do not compare react nodes
  if (React.isValidElement(a) || React.isValidElement(b)) {
    return a === b;
  }

  if (a && b && _isFunction(a.equals)) {
    return a.equals(b);
  }
};

export var isEqualProps = function isEqualProps(a, b) {
  return _isEqualWith(a, b, isEqualPropsCustomizer);
};

export var getChangedProps = function getChangedProps(prev, next) {
  var prevKeys = _keys(prev);
  var nextKeys = _keys(next);
  var allKeys = _union(prevKeys, nextKeys);
  var hasDiff = false;
  var diff = {};

  allKeys.forEach(function (key) {
    if (!isEqualProps(prev[key], next[key])) {
      hasDiff = true;

      diff[key] = next[key];
    }
  }, {});

  return hasDiff ? diff : null;
};

export var createListener = function createListener(getHandler) {
  return function (x) {
    var fn = getHandler();

    if (fn) {
      fn(x);
    }
  };
};

export var createListeners = function createListeners(events, getHandler) {
  return _keys(events).map(function (handler) {
    var event = events[handler];

    return [event, createListener(function () {
      return getHandler(handler);
    })];
  });
};