import _pick from "lodash-es/pick";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from "react";

import PropTypes from "prop-types";
import MarkerEvents from "./MarkerEvents";
import { MarkerContext } from "./MarkerContext";
import { createListeners, getChangedProps } from "../internal/Utils";
import { MapContext } from "../internal/MapContext";

var pickProps = function pickProps(x) {
  return _pick(x, ["position", "title", "visible", "clickable", "draggable", "crossOnDrag", "animation", "cursor", "icon", "label", "opacity", "optimized", "shape", "zIndex"]);
};

/**
 * Draws `google.maps.Marker`.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, Marker } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <Marker position={props.center} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.Marker](https://developers.google.com/maps/documentation/javascript/reference#Marker)
 * * [google.maps.MarkerOptions](https://developers.google.com/maps/documentation/javascript/reference#MarkerOptions)
 */
export var Marker = function (_React$Component) {
  _inherits(Marker, _React$Component);

  function Marker() {
    _classCallCheck(this, Marker);

    return _possibleConstructorReturn(this, (Marker.__proto__ || Object.getPrototypeOf(Marker)).apply(this, arguments));
  }

  _createClass(Marker, [{
    key: "getChildContext",
    value: function getChildContext() {
      return { markerContext: this.markerContext };
    }
  }, {
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var mapContext = this.context.mapContext;


      var marker = new mapContext.maps.Marker();
      var options = this.getOptions(this.props);

      marker.setValues(options);
      marker.setMap(mapContext.map);

      this.marker = marker;
      this.markerContext = new MarkerContext(marker);

      marker.addListener(MarkerEvents.onDragEnd, function () {
        marker.setPosition(_this2.props.position);
      });

      createListeners(MarkerEvents, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        marker.addListener(event, listener);
      });
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var next = this.getOptions(nextProps);
      var prev = this.getOptions(this.props);
      var diff = getChangedProps(prev, next);

      if (diff) {
        this.marker.setValues(diff);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.marker.setMap(null);
      this.context.mapContext.maps.event.clearInstanceListeners(this.marker);
    }
  }, {
    key: "getOptions",
    value: function getOptions(props) {
      var mapContext = this.context.mapContext;

      var options = pickProps(props);

      options.animation = mapContext.getEnum("Animation", options.animation);

      if (React.isValidElement(options.icon)) {
        delete options.icon;
      }

      return options;
    }
  }, {
    key: "render",
    value: function render() {
      return React.isValidElement(this.props.icon) ? this.props.icon : null;
    }
  }]);

  return Marker;
}(React.Component);

Marker.childContextTypes = {
  markerContext: PropTypes.instanceOf(MarkerContext).isRequired
};

Marker.contextTypes = {
  mapContext: PropTypes.instanceOf(MapContext).isRequired
};

Marker.defaultProps = {
  visible: true,
  clickable: true,
  optimized: true,
  draggable: false,
  crossOnDrag: true,
  animation: "NONE"
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  Marker.propTypes = {
    /**
     * Marker position.
     */
    position: PropTypes.shape({
      /**
       * The latitude in degrees.
       */
      lat: PropTypes.number.isRequired,
      /**
       * The longitude in degrees.
       */
      lng: PropTypes.number.isRequired
    }).isRequired,

    /**
     * Rollover text.
     */
    title: PropTypes.string,

    /**
     * If `true`, the marker is visible.
     */
    visible: PropTypes.bool,

    /**
     * If `true`, the marker receives mouse and touch events.
     */
    clickable: PropTypes.bool,

    /**
     * If `true`, the marker can be dragged.
     */
    draggable: PropTypes.bool,

    /**
     * If `false`, disables cross that appears beneath the marker when dragging.
     */
    crossOnDrag: PropTypes.bool,

    //
    // Style
    //

    /**
     * Which animation to play when marker is added to a map.
     */
    animation: PropTypes.oneOf(["NONE", "BOUNCE", "DROP"]),

    /**
     * Mouse cursor to show on hover.
     */
    cursor: PropTypes.string,

    /**
     * Icon for the foreground.
     */
    icon: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),

    /**
     * Adds a label to the marker.
     *
     * The label can either be a `string`, or a [google.maps.MarkerLabel](https://developers.google.com/maps/documentation/javascript/3.exp/reference#MarkerLabel) object.
     */
    label: PropTypes.oneOfType([PropTypes.string, PropTypes.shape({
      /**
       * The color of the label text.
       *
       * Default color is black.
       */
      color: PropTypes.string,
      /**
       * The font family of the label text.
       *
       * Equivalent to the CSS font-family property.
       */
      fontFamily: PropTypes.string,
      /**
       * The font size of the label text.
       *
       * Equivalent to the CSS font-size property.
       *
       * Default size is 14px.
       */
      fontSize: PropTypes.string,
      /**
       * The font weight of the label text
       *
       * Equivalent to the CSS font-weight property.
       */
      fontWeight: PropTypes.string,
      /**
       * The text to be displayed in the label.
       */
      text: PropTypes.string
    })]),

    /**
     * The marker's opacity between 0.0 and 1.0.
     */
    opacity: PropTypes.number,

    /**
     * Optimization renders many markers as a single static element.
     *
     * Disable optimized rendering for animated GIFs or PNGs,
     * or when each marker must be rendered as a separate DOM element
     * (advanced usage only).
     */
    optimized: PropTypes.bool,

    /**
     * Image map region definition used for drag/click.
     *
     * See also: [google.maps.MarkerShape](https://developers.google.com/maps/documentation/javascript/3.exp/reference#MarkerShape)
     */
    shape: PropTypes.shape({
      /**
       * Specifies the pixel position of the shape
       * relative to the top-left corner
       * of the target image.
       */
      coords: PropTypes.arrayOf(PropTypes.number),

      /**
       * Describes the shape's type.
       */
      type: PropTypes.oneOf(["circle", "poly", "rect"])
    }),

    /**
     * All markers are displayed on the map in order of their zIndex,
     * with higher values displaying in front of markers with lower values.
     * By default, markers are displayed according to their vertical position on screen,
     * with lower markers appearing in front of markers further up the screen.
     */
    zIndex: PropTypes.number,

    //
    // Events
    //

    /**
     * This handlers is called when the marker icon was clicked.
     */
    onClick: PropTypes.func,
    /**
     * This handlers is called when the marker icon was double clicked.
     */
    onDoubleClick: PropTypes.func,
    /**
     * This handlers is called when the marker icon was clicked.
     */
    onRightClick: PropTypes.func,

    /**
     * This handlers is called for a mouse down on the marker.
     */
    onMouseDown: PropTypes.func,
    /**
     * This handlers is called when the mouse leaves the area of the marker icon.
     */
    onMouseOut: PropTypes.func,
    /**
     * This handlers is called when the mouse enters the area of the marker icon.
     */
    onMouseOver: PropTypes.func,
    /**
     * This handlers is called when for a mouse up on the marker.
     */
    onMouseUp: PropTypes.func,

    /**
     * This handlers is called when the marker icon was clicked.
     */
    onDrag: PropTypes.func,
    /**
     * This handlers is called when the marker icon was clicked.
     */
    onDragStart: PropTypes.func,
    /**
     * This handlers is called when the marker icon was clicked.
     */
    onDragEnd: PropTypes.func,

    /**
     * This handlers is called when the marker `position` property changes.
     */
    onPositionChanged: PropTypes.func
  };
}