import _pick from "lodash-es/pick";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from "react";

import PropTypes from "prop-types";
import GoogleMarkerClusterer from "@google/markerclustererplus";
import MarkerEvents from "./MarkerEvents";
import { createListeners } from "../internal/Utils";
import { MapContext } from "../internal/MapContext";

var pickProps = function pickProps(x) {
  return _pick(x, ["maxZoom", "gridSize", "styles", "imagePath", "zoomOnClick", "averageCenter", "imageExtension", "minimumClusterSize"]);
};

/**
 * Draws `google.maps.MarkerClusterer`.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, MarkerClusterer } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   const markers = [
 *     new LatLng(),
 *     { lat: ..., lng: ... },
 *     new Marker({ position: new LatLng() })
 *   ];
 *
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <MarkerClusterer markers={props.center} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [@google/clustererplus](https://github.com/googlemaps/v3-utility-library/tree/master/markerclustererplus)
 * * [@google/clustererplus#Options](https://googlemaps.github.io/js-marker-clusterer/docs/reference.html)
 */
export var MarkerClusterer = function (_React$Component) {
  _inherits(MarkerClusterer, _React$Component);

  function MarkerClusterer() {
    _classCallCheck(this, MarkerClusterer);

    return _possibleConstructorReturn(this, (MarkerClusterer.__proto__ || Object.getPrototypeOf(MarkerClusterer)).apply(this, arguments));
  }

  _createClass(MarkerClusterer, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var mapContext = this.context.mapContext;


      var options = pickProps(this.props);
      var markers = this.getMarkers(this.props.markers);

      var markerClusterer = new GoogleMarkerClusterer(mapContext.map, markers, options);

      this.markerClusterer = markerClusterer;

      createListeners(MarkerEvents, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        markerClusterer.addListener(event, listener);
      });
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.markerClusterer.setMap(null);
      this.markerClusterer.clearMarkers();
      this.context.mapContext.maps.event.clearInstanceListeners(this.markerClusterer);
    }
  }, {
    key: "getMarkers",
    value: function getMarkers() {
      var _this3 = this;

      var markers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var mapContext = this.context.mapContext;


      return markers.reduce(function (acc, item) {
        if (_this3.isMarker(item)) {
          acc.push(item);
        }

        if (_this3.isLatLng(item)) {
          acc.push(new mapContext.maps.Marker({ position: item }));
        }

        if (!_this3.isMarker(item) && !_this3.isLatLng(item)) {
          var latLng = new mapContext.maps.LatLng(item.lat, item.lng);

          acc.push(new mapContext.maps.Marker({ position: latLng }));
        }

        return acc;
      }, []);
    }
  }, {
    key: "isMarker",
    value: function isMarker(item) {
      var mapContext = this.context.mapContext;


      return item instanceof mapContext.maps.Marker;
    }
  }, {
    key: "isLatLng",
    value: function isLatLng(item) {
      var mapContext = this.context.mapContext;


      return item instanceof mapContext.maps.LatLng;
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return MarkerClusterer;
}(React.Component);

MarkerClusterer.contextTypes = {
  mapContext: PropTypes.instanceOf(MapContext).isRequired
};

MarkerClusterer.defaultProps = {
  markers: []
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  MarkerClusterer.propTypes = {
    /**
     * Markers to add to the cluster
     */
    markers: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.shape({
      lat: PropTypes.number,
      lng: PropTypes.number
    }), PropTypes.shape({
      position: PropTypes.shape({
        lat: PropTypes.number,
        lng: PropTypes.number
      })
    })])),

    /**
     * The grid size of a cluster in pixels.
     */
    gridSize: PropTypes.number,

    /**
     * The maximum zoom level that a marker can be part of a cluster.
     */
    maxZoom: PropTypes.number,

    /**
     * Whether the default behaviour of clicking on a cluster is to zoom into it.
     */
    zoomOnClick: PropTypes.bool,

    /**
     * The base URL where the images representing clusters will be found. The full URL will be: {imagePath}[1-5].{imageExtension}
     Default: '../images/m'.
     */
    imagePath: PropTypes.string,

    /**
     * The suffix for images URL representing clusters will be found. See _imagePath_ for details.
     * Default: 'png'.
     */
    imageExtension: PropTypes.string,

    /**
     * Whether the center of each cluster should be the average of all markers in the cluster.
     */
    averageCenter: PropTypes.bool,

    /**
     * The minimum number of markers to be in a cluster before the markers are hidden and a count is shown.
     */
    minimumClusterSize: PropTypes.number,

    /**
     * Style properties.
     */
    styles: PropTypes.shape({
      /**
       * The image url.
       */
      url: PropTypes.string,

      /**
       * The image height.
       */
      height: PropTypes.number,

      /**
       * The image width.
       */
      width: PropTypes.number,

      /**
       * The anchor position of the label text.
       */
      anchor: PropTypes.array,

      /**
       * The text color.
       */
      textColor: PropTypes.string,

      /**
       * The text size.
       */
      textSize: PropTypes.number,

      /**
       * The position of the backgound x, y.
       */
      backgroundPosition: PropTypes.string,

      /**
       * The anchor position of the icon x, y.
       */
      iconAnchor: PropTypes.array
    }),

    //
    // Events
    //

    /**
     * This handlers is called when the markerClusterer icon was clicked.
     */
    onClick: PropTypes.func,
    /**
     * This handlers is called when the mouse leaves the area of the markerClusterer icon.
     */
    onMouseOut: PropTypes.func,
    /**
     * This handlers is called when the mouse enters the area of the markerClusterer icon.
     */
    onMouseOver: PropTypes.func
  };
}