import _pick from "lodash-es/pick";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from "react";

import PropTypes from "prop-types";
import { createListeners, getChangedProps } from "../internal/Utils";
import { MapContext } from "../internal/MapContext";
import GenericEvents from "../internal/GenericEvents";

var pickProps = function pickProps(x) {
  return _pick(x, ["path", "icons", "zIndex", "visible", "geodesic", "clickable", "draggable", "strokeColor", "strokeWeight", "strokeOpacity"]);
};

/**
 * Draws `google.maps.Polyline`.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, Polyline } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <Polyline path={[props.start, props.center, props.end]} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.Polyline](https://developers.google.com/maps/documentation/javascript/reference#Polyline)
 * * [google.maps.PolylineOptions](https://developers.google.com/maps/documentation/javascript/reference#PolylineOptions)
 */
export var Polyline = function (_React$Component) {
  _inherits(Polyline, _React$Component);

  function Polyline() {
    _classCallCheck(this, Polyline);

    return _possibleConstructorReturn(this, (Polyline.__proto__ || Object.getPrototypeOf(Polyline)).apply(this, arguments));
  }

  _createClass(Polyline, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var mapContext = this.context.mapContext;


      var polyline = new mapContext.maps.Polyline();
      var options = pickProps(this.props);

      this.polyline = polyline;

      polyline.setValues(options);
      polyline.setMap(mapContext.map);

      polyline.addListener(GenericEvents.onDragEnd, function (event) {
        var path = polyline.getPath();

        if (path) {
          // eslint-disable-next-line no-param-reassign
          event.path = path;
        }

        polyline.setPath(_this2.props.path);
      });

      createListeners(GenericEvents, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        polyline.addListener(event, listener);
      });
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var next = pickProps(nextProps);
      var prev = pickProps(this.props);
      var diff = getChangedProps(prev, next);

      if (diff) {
        this.polyline.setValues(diff);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.polyline.setMap(null);
      this.context.mapContext.maps.event.clearInstanceListeners(this.polyline);
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return Polyline;
}(React.Component);

Polyline.contextTypes = {
  mapContext: PropTypes.instanceOf(MapContext).isRequired
};

Polyline.defaultProps = {
  draggable: false,
  clickable: true,
  geodesic: false,
  visible: true
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  Polyline.propTypes = {
    /**
     * If set to true, the user can drag this shape over the map.
     *
     * The `geodesic` property defines the mode of dragging.
     */
    draggable: PropTypes.bool,

    /**
     * Indicates whether this `Polyline` handles mouse events.
     */
    clickable: PropTypes.bool,

    /**
     * When `true`, edges of the polygon are interpreted as geodesic and will follow the curvature of the Earth.
     * When `false`, edges of the polygon are rendered as straight lines in screen space.
     *
     * Note that the shape of a geodesic polygon may appear to change when dragged,
     * as the dimensions are maintained relative to the surface of the earth.
     */
    geodesic: PropTypes.bool,

    /**
     * Whether this `Polyline` is visible on the map.
     */
    visible: PropTypes.bool,

    /**
     * The ordered sequence of coordinates of the `Polyline`.
     */
    path: PropTypes.arrayOf(PropTypes.shape({
      /**
       * The latitude in degrees.
       */
      lat: PropTypes.number.isRequired,

      /**
       * The longitude in degrees.
       */
      lng: PropTypes.number.isRequired
    })).isRequired,

    //
    // Style
    //

    /**
     * The stroke color.
     *
     * All CSS3 colors are supported except for extended named colors.
     */
    strokeColor: PropTypes.string,

    /**
     * The stroke opacity between 0.0 and 1.0.
     */
    strokeOpacity: PropTypes.number,

    /**
     * The stroke width in pixels.
     */
    strokeWeight: PropTypes.number,

    /**
     * The zIndex compared to other polys.
     */
    zIndex: PropTypes.number,

    //
    // Events
    //

    /**
     * This handler is fired when the DOM `click` handlers is fired
     * on the `Polyline`.
     */
    onClick: PropTypes.func,
    /**
     * This handler is fired when the DOM `dblclick` handlers is fired
     * on the `Polyline`.
     */
    onDoubleClick: PropTypes.func,
    /**
     * This handler is fired when the `Polyline` is right-clicked on.
     */
    onRightClick: PropTypes.func,

    /**
     * This handler is fired on `mouseout` handlers is fired on `Polyline`.
     */
    onMouseOut: PropTypes.func,
    /**
     * This handler is fired on `mouseover` handlers is fired on `Polyline`.
     */
    onMouseOver: PropTypes.func,

    /**
     * This handler is fired when the DOM `mousemove` handlers is fired
     * on `Polyline`.
     */
    onMouseMove: PropTypes.func,
    /**
     * This handler is fired when the DOM `mousedown` handlers is fired
     * on `Polyline`.
     */
    onMouseDown: PropTypes.func,
    /**
     * This handler is fired when the DOM `mouseup` handlers is fired
     * on `Polyline`.
     */
    onMouseUp: PropTypes.func,

    /**
     * This handler is repeatedly fired while the user drags the `Polyline`.
     */
    onDrag: PropTypes.func,
    /**
     * This handler is fired when the user starts dragging the `Polyline`.
     */
    onDragStart: PropTypes.func,
    /**
     * This handler is fired when the user stops dragging the `Polyline`.
     */
    onDragEnd: PropTypes.func
  };
}