"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PanToBounds = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Utils = require("../internal/Utils");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Pans the map by the minimum amount necessary to contain the given `LatLngBounds`.
 * It makes no guarantee where on the map the bounds will be, except that as much of the
 * bounds as possible will be visible.
 *
 * The bounds will be positioned inside the area bounded by the map type and navigation
 * (pan, zoom, and Street View) controls, if they are present on the map.
 *
 * If the bounds is larger than the map, the map will be shifted to include the northwest
 * corner of the bounds.
 *
 * If the change in the map's position is less than both the width and height of the map,
 * the transition will be smoothly animated.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, PanToBounds } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <PanToBounds latLngBounds={props.bounds} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.Map](https://developers.google.com/maps/documentation/javascript/reference#Map)
 */
var PanToBounds = exports.PanToBounds = function (_React$Component) {
  _inherits(PanToBounds, _React$Component);

  function PanToBounds() {
    _classCallCheck(this, PanToBounds);

    return _possibleConstructorReturn(this, (PanToBounds.__proto__ || Object.getPrototypeOf(PanToBounds)).apply(this, arguments));
  }

  _createClass(PanToBounds, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      this.animate();
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps) {
      return !(0, _Utils.isEqualProps)(this.props.latLngBounds, nextProps.latLngBounds);
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      this.animate(nextProps);
    }
  }, {
    key: "animate",
    value: function animate() {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props,
          latLngBounds = _ref.latLngBounds;

      var mapContext = this.context.mapContext;


      mapContext.map.panToBounds(mapContext.createLatLngBounds(latLngBounds));
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return PanToBounds;
}(_react2.default.Component);

PanToBounds.contextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext).isRequired
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  PanToBounds.propTypes = {
    /**
     * Array of positions that will be used to generate `LatLngBounds` object.
     *
     * See also: [google.maps.LatLngBounds](https://developers.google.com/maps/documentation/javascript/3.exp/reference#LatLngBounds)
     */
    latLngBounds: _propTypes2.default.arrayOf(_propTypes2.default.shape({
      /**
       * The latitude in degrees.
       */
      lat: _propTypes2.default.number.isRequired,
      /**
       * The longitude in degrees.
       */
      lng: _propTypes2.default.number.isRequired
    })).isRequired
  };
}