"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FullscreenControl = FullscreenControl;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Control = require("./Control");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Controls display options of fullscreen control.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, FullscreenControl } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <FullscreenControl />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.FullscreenControlOptions](https://developers.google.com/maps/documentation/javascript/reference#FullscreenControlOptions)
 */
function FullscreenControl(props, context) {
  return _react2.default.createElement(_Control.Control, {
    name: "fullscreenControl",
    options: {
      position: context.mapContext.getEnum("ControlPosition", props.position)
    }
  });
}

FullscreenControl.contextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext).isRequired
};

FullscreenControl.defaultProps = {
  position: "RIGHT_TOP"
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  FullscreenControl.propTypes = {
    /**
     * Position id.
     *
     * Used to specify the position of the control on the map.
     *
     * See also: [google.maps.ControlPosition](https://developers.google.com/maps/documentation/javascript/reference#ControlPosition)
     */
    position: _propTypes2.default.oneOf(["BOTTOM_CENTER", "BOTTOM_LEFT", "BOTTOM_RIGHT", "LEFT_BOTTOM", "LEFT_CENTER", "LEFT_TOP", "RIGHT_BOTTOM", "RIGHT_CENTER", "RIGHT_TOP", "TOP_CENTER", "TOP_LEFT", "TOP_RIGHT"])
  };
}