"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DrawingControl = undefined;

var _map2 = require("lodash/map");

var _map3 = _interopRequireDefault(_map2);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _DrawingControlEvents = require("./DrawingControlEvents");

var _DrawingControlEvents2 = _interopRequireDefault(_DrawingControlEvents);

var _Utils = require("../internal/Utils");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Controls display options of `google.maps.drawing.DrawingManager` control.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, DrawingControl } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <DrawingControl onOverlayComplete={props.onOverlayComplete} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.drawing.DrawingManager](https://developers.google.com/maps/documentation/javascript/3.exp/reference#DrawingManager)
 * * [google.maps.drawing.DrawingManagerOptions](https://developers.google.com/maps/documentation/javascript/3.exp/reference#DrawingManagerOptions)
 */
var DrawingControl = exports.DrawingControl = function (_React$Component) {
  _inherits(DrawingControl, _React$Component);

  function DrawingControl() {
    _classCallCheck(this, DrawingControl);

    return _possibleConstructorReturn(this, (DrawingControl.__proto__ || Object.getPrototypeOf(DrawingControl)).apply(this, arguments));
  }

  _createClass(DrawingControl, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var mapContext = this.context.mapContext;


      var options = this.getOptions();
      var drawingManager = new mapContext.maps.drawing.DrawingManager();

      this.drawingManager = drawingManager;

      drawingManager.setValues(options);
      drawingManager.setMap(mapContext.map);

      drawingManager.addListener(_DrawingControlEvents2.default.onOverlayComplete, function (x) {
        x.overlay.setMap(null);
      });

      (0, _Utils.createListeners)(_DrawingControlEvents2.default, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        drawingManager.addListener(event, listener);
      });
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var prevOptions = this.getOptions();
      var nextOptions = this.getOptions(nextProps);

      if (!(0, _Utils.isEqualProps)(prevOptions, nextOptions)) {
        this.drawingManager.setValues(nextOptions);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.drawingManager.setMap(null);
    }
  }, {
    key: "getOptions",
    value: function getOptions() {
      var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props,
          position = _ref3.position,
          drawingModes = _ref3.drawingModes;

      var mapContext = this.context.mapContext;


      return {
        drawingControl: true,
        drawingControlOptions: {
          position: mapContext.getEnum("ControlPosition", position),
          drawingModes: (0, _map3.default)(drawingModes, function (x) {
            return mapContext.getEnum("drawing.OverlayType", x);
          })
        }
      };
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return DrawingControl;
}(_react2.default.Component);

DrawingControl.contextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext).isRequired
};

DrawingControl.defaultProps = {
  position: "TOP_LEFT",
  drawingModes: ["circle", "marker", "polygon", "polyline", "rectangle"]
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  DrawingControl.propTypes = {
    /**
     * Position id.
     *
     * Used to specify the position of the control on the map.
     *
     * See also: [google.maps.ControlPosition](https://developers.google.com/maps/documentation/javascript/reference#ControlPosition)
     */
    position: _propTypes2.default.oneOf(["BOTTOM_CENTER", "BOTTOM_LEFT", "BOTTOM_RIGHT", "LEFT_BOTTOM", "LEFT_CENTER", "LEFT_TOP", "RIGHT_BOTTOM", "RIGHT_CENTER", "RIGHT_TOP", "TOP_CENTER", "TOP_LEFT", "TOP_RIGHT"]),

    /**
     * The drawing modes to display in the drawing control, in the order in
     * which they are to be displayed.
     *
     * The hand icon (which corresponds to the null drawing mode)
     * is always available and is not to be specified in this array.
     */
    drawingModes: _propTypes2.default.arrayOf(_propTypes2.default.oneOf(["circle", "marker", "polygon", "polyline", "rectangle"])),

    /**
     * This handler is called when the user has finished drawing a `circle`.
     */
    onCircleComplete: _propTypes2.default.func,

    /**
     * This handler is called when the user has finished drawing a `marker`.
     */
    onMarkerComplete: _propTypes2.default.func,

    /**
     * This handler is called when the user has finished drawing an `overlay`
     * of any type.
     */
    onOverlayComplete: _propTypes2.default.func,

    /**
     * This handler is called when the user has finished drawing a `polygon`.
     */
    onPolygonComplete: _propTypes2.default.func,

    /**
     * This handler is called when the user has finished drawing a `polyline`.
     */
    onPolylineComplete: _propTypes2.default.func,

    /**
     * This handler is called when the user has finished drawing a `rectangle`.
     */
    onRectangleComplete: _propTypes2.default.func
  };
}