"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GoogleMap = undefined;

var _pick2 = require("lodash/pick");

var _pick3 = _interopRequireDefault(_pick2);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _GoogleMapEvents = require("./GoogleMapEvents");

var _GoogleMapEvents2 = _interopRequireDefault(_GoogleMapEvents);

var _Utils = require("../internal/Utils");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var styles = { map: { height: "100%" } };

var pickProps = function pickProps(x) {
  return (0, _pick3.default)(x, ["zoom", "center", "mapTypeId", "clickableIcons", "backgroundColor", "disableDoubleClickZoom"]);
};

/**
 * Draws `google.maps.Map`.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return <GoogleMap {...props} maps={google.maps} />
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.Map](https://developers.google.com/maps/documentation/javascript/reference#Map)
 * * [google.maps.MapOptions](https://developers.google.com/maps/documentation/javascript/reference#MapOptions)
 */

var GoogleMap = exports.GoogleMap = function (_React$Component) {
  _inherits(GoogleMap, _React$Component);

  function GoogleMap(props, context) {
    _classCallCheck(this, GoogleMap);

    var _this = _possibleConstructorReturn(this, (GoogleMap.__proto__ || Object.getPrototypeOf(GoogleMap)).call(this, props, context));

    _this.map = null;
    _this.node = null;
    _this.maps = props.maps;
    _this.mapContext = null;
    return _this;
  }

  _createClass(GoogleMap, [{
    key: "getChildContext",
    value: function getChildContext() {
      return { mapContext: this.mapContext };
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;

      this.map = new this.maps.Map(this.node);
      this.mapContext = new _MapContext.MapContext(this.map, this.maps);

      var options = this.getOptions(this.props);

      options.disableDefaultUI = true;

      this.map.setValues(options);

      (0, _Utils.createListeners)(_GoogleMapEvents2.default, function (x) {
        return _this2.props[x];
      }).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            event = _ref2[0],
            listener = _ref2[1];

        switch (event) {
          case _GoogleMapEvents2.default.onBoundsChanged:
            _this2.map.addListener(event, function () {
              return listener({ bounds: _this2.map.getBounds() });
            });
            break;

          case _GoogleMapEvents2.default.onZoomChanged:
            _this2.map.addListener(event, function () {
              return listener({ zoom: _this2.map.getZoom() });
            });
            break;

          default:
            _this2.map.addListener(event, listener);
        }
      });

      this.forceUpdate();
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps) {
      var prevOptions = this.getOptions(prevProps);
      var nextOptions = this.getOptions(this.props);
      var options = (0, _Utils.getChangedProps)(prevOptions, nextOptions);

      if (options) {
        this.map.setValues(options);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.maps.event.clearInstanceListeners(this.map);
    }
  }, {
    key: "getOptions",
    value: function getOptions(props) {
      var options = pickProps(props);

      options.mapTypeId = this.mapContext.getEnum("MapTypeId", options.mapTypeId);

      return options;
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;

      var _props = this.props,
          style = _props.style,
          className = _props.className;


      return _react2.default.createElement(
        "div",
        { style: style, className: className },
        _react2.default.createElement("div", { style: styles.map, ref: function ref(x) {
            return _this3.node = x;
          } }),
        Boolean(this.mapContext) && this.props.children
      );
    }
  }]);

  return GoogleMap;
}(_react2.default.Component);

GoogleMap.childContextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext)
};

GoogleMap.defaultProps = {
  mapTypeId: "ROADMAP",

  clickableIcons: true,
  disableDoubleClickZoom: false
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  GoogleMap.propTypes = {
    /**
     * Loaded `google.Maps` instance.
     */
    maps: _propTypes2.default.object.isRequired,

    /**
     * The initial Map `center`.
     */
    center: _propTypes2.default.shape({
      /**
       * The latitude in degrees.
       */
      lat: _propTypes2.default.number.isRequired,
      /**
       * The longitude in degrees.
       */
      lng: _propTypes2.default.number.isRequired
    }).isRequired,

    /**
     * The initial Map `zoom` level.
     */
    zoom: _propTypes2.default.number.isRequired,

    /**
     * The initial Map `mapTypeId`.
     */
    mapTypeId: _propTypes2.default.oneOf(["HYBRID", "ROADMAP", "SATELLITE", "TERRAIN"]),

    /**
     * Google Maps child components.
     */
    children: _propTypes2.default.node,

    /**
     * When `false`, map icons are not clickable.
     *
     * A map icon represents a point of interest, also known as a POI.
     */
    clickableIcons: _propTypes2.default.bool,

    /**
     * Enables/disables `zoom` and `center` on double click.
     */
    disableDoubleClickZoom: _propTypes2.default.bool,

    //
    // Styling
    //

    /**
     * Styles of map div.
     */
    style: _propTypes2.default.object,

    /**
     * Classes of map div.
     */
    className: _propTypes2.default.string,

    /**
     * Color used for the background of the Map.
     *
     * This color will be visible when tiles have not yet
     * loaded as the user pans.
     */
    backgroundColor: _propTypes2.default.string,

    //
    // Events
    //

    /**
     * This handler is called when the user clicks on the map.
     *
     * An `ApiMouseEvent` with properties for the clicked location is returned
     * unless a place icon was clicked, in which case an `IconMouseEvent`
     * with a `placeid` is returned.
     *
     * `IconMouseEvent` and `ApiMouseEvent` are identical, except that
     * `IconMouseEvent` has the `placeid` field.
     *
     * The event can always be treated as an `ApiMouseEvent` when the
     * `placeid` is not important.
     *
     * The click event is not fired if a `Marker` or `InfoWindow` was clicked.
     */
    onClick: _propTypes2.default.func,

    /**
     * This handler is called when the user double-clicks on the map.
     *
     * Note that the `onClick` handler will be also called,
     * right before this one.
     */
    onDoubleClick: _propTypes2.default.func,

    /**
     * This handler is called when the DOM context menu event is
     * fired on the map container.
     */
    onRightClick: _propTypes2.default.func,

    /**
     * This handler is called whenever the user's mouse moves over
     * the map container.
     */
    onMouseMove: _propTypes2.default.func,

    /**
     * This handler is called when the user's mouse exits the map container.
     */
    onMouseOut: _propTypes2.default.func,

    /**
     * This handler is called when the user's mouse enters the map container.
     */
    onMouseOver: _propTypes2.default.func,

    /**
     * This handler is repeatedly called while the user drags the map.
     */
    onDrag: _propTypes2.default.func,

    /**
     * This handler is called when the user stops dragging the map.
     */
    onDragEnd: _propTypes2.default.func,

    /**
     * This handler is called when the user starts dragging the map.
     */
    onDragStart: _propTypes2.default.func,

    /**
     * This handler is called when the map becomes idle after panning or zooming.
     */
    onIdle: _propTypes2.default.func,

    /**
     * This handler is called when the visible tiles have finished loading.
     */
    onTilesLoaded: _propTypes2.default.func,

    /**
     * This handler is called when the map `tilt` property changes.
     */
    onTiltChanged: _propTypes2.default.func,

    /**
     * This event is fired when the map `zoom` property changes.
     */
    onZoomChanged: _propTypes2.default.func,

    /**
     * This handler is called when the viewport bounds have changed.
     */
    onBoundsChanged: _propTypes2.default.func,

    /**
     * This handler is called when the map `center` property changes.
     */
    onCenterChanged: _propTypes2.default.func,

    /**
     * This handler is called when the map `heading` property changes.
     */
    onHeadingChanged: _propTypes2.default.func,

    /**
     * This handler is called when the `mapTypeId` property changes.
     */
    onMapTypeIdChanged: _propTypes2.default.func,

    /**
     * This handler is called when the `projection` has changed.
     */
    onProjectionChanged: _propTypes2.default.func
  };
}