"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HeatmapLayer = undefined;

var _pick2 = require("lodash/pick");

var _pick3 = _interopRequireDefault(_pick2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Utils = require("../internal/Utils");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var pickProps = function pickProps(x) {
  return (0, _pick3.default)(x, ["data", "dissipating", "gradient", "maxIntensity", "opacity", "radius"]);
};

/**
 * Draws `google.maps.visualization.HeatmapLayer`.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, HeatmapLayer } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <HeatmapLayer data={props.heatMapData} />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.visualization.HeatmapLayer](https://developers.google.com/maps/documentation/javascript/reference/visualization#HeatmapLayer)
 * * [google.maps.visualization.HeatmapLayerOptions](https://developers.google.com/maps/documentation/javascript/reference/visualization#HeatmapLayerOptions)
 */

var HeatmapLayer = exports.HeatmapLayer = function (_React$Component) {
  _inherits(HeatmapLayer, _React$Component);

  function HeatmapLayer() {
    _classCallCheck(this, HeatmapLayer);

    return _possibleConstructorReturn(this, (HeatmapLayer.__proto__ || Object.getPrototypeOf(HeatmapLayer)).apply(this, arguments));
  }

  _createClass(HeatmapLayer, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var mapContext = this.context.mapContext;


      var heatmapLayer = new mapContext.maps.visualization.HeatmapLayer();
      var options = this.getOptions(this.props);

      heatmapLayer.setValues(options);
      heatmapLayer.setMap(mapContext.map);

      this.heatmapLayer = heatmapLayer;
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var next = this.getOptions(nextProps);
      var prev = this.getOptions(this.props);
      var diff = (0, _Utils.getChangedProps)(prev, next);

      if (diff) {
        this.heatmapLayer.setValues(diff);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.heatmapLayer.setMap(null);
    }

    /* eslint-disable-next-line class-methods-use-this */

  }, {
    key: "getOptions",
    value: function getOptions(props) {
      return pickProps(props);
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return HeatmapLayer;
}(_react2.default.Component);

HeatmapLayer.contextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext).isRequired
};

HeatmapLayer.defaultProps = {
  opacity: 0.8
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  HeatmapLayer.propTypes = {
    /**
     * The data points to display.
     */
    data: _propTypes2.default.arrayOf(_propTypes2.default.object, _propTypes2.default.shape({
      lat: _propTypes2.default.number,
      lng: _propTypes2.default.number
    })).isRequired,

    /**
     * Specifies whether heatmaps dissipate on zoom.
     */
    dissipating: _propTypes2.default.bool,

    /**
     * The color gradient of the heatmap, specified as an array of CSS color strings. All CSS3 colors are supported except for extended named colors.
     */
    gradient: _propTypes2.default.arrayOf(_propTypes2.default.string),

    /**
     * The maximum intensity of the heatmap.
     */
    maxIntensity: _propTypes2.default.number,

    /**
     * The opacity of the heatmap, expressed as a number between 0 and 1
     */
    opacity: _propTypes2.default.number,

    /**
     * The radius of influence for each data point, in pixels.
     */
    radius: _propTypes2.default.number
  };
}