"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MarkerIcon = undefined;

var _isEqual2 = require("lodash/isEqual");

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _pick2 = require("lodash/pick");

var _pick3 = _interopRequireDefault(_pick2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _MarkerContext = require("./MarkerContext");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var pickProps = function pickProps(x) {
  return (0, _pick3.default)(x, ["url", "anchor", "labelOrigin", "origin", "size", "scaledSize"]);
};

/**
 * Defines icon options of `Marker` component.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, Marker, MarkerIcon } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <Marker
 *         position={props.center}
 *         icon={<MarkerIcon url={props.markerIcon} />}
 *       />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.Icon](https://developers.google.com/maps/documentation/javascript/reference#Icon)
 */

var MarkerIcon = exports.MarkerIcon = function (_React$Component) {
  _inherits(MarkerIcon, _React$Component);

  function MarkerIcon() {
    _classCallCheck(this, MarkerIcon);

    return _possibleConstructorReturn(this, (MarkerIcon.__proto__ || Object.getPrototypeOf(MarkerIcon)).apply(this, arguments));
  }

  _createClass(MarkerIcon, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _context = this.context,
          mapContext = _context.mapContext,
          marker = _context.markerContext.marker;


      marker.setIcon(mapContext.createIcon(this.props));
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var _context2 = this.context,
          mapContext = _context2.mapContext,
          markerContext = _context2.markerContext;


      var nextIcon = pickProps(nextProps);
      var prevIcon = pickProps(this.props);

      if (!(0, _isEqual3.default)(prevIcon, nextIcon)) {
        markerContext.marker.setIcon(mapContext.createIcon(nextIcon));
      }
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return MarkerIcon;
}(_react2.default.Component);

MarkerIcon.contextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext).isRequired,
  markerContext: _propTypes2.default.instanceOf(_MarkerContext.MarkerContext).isRequired
};

MarkerIcon.defaultProps = {
  origin: { x: 0, y: 0 }
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  MarkerIcon.propTypes = {
    /**
     * The URL of the image or sprite sheet.
     */
    url: _propTypes2.default.string.isRequired,

    /**
     * The position at which to anchor an image in correspondence to the location of the marker on the map.
     *
     * By default, the anchor is located along the center point of the bottom of the image.
     */
    anchor: _propTypes2.default.shape({
      /**
       * The X coordinate.
       */
      x: _propTypes2.default.number.isRequired,
      /**
       * The Y coordinate.
       */
      y: _propTypes2.default.number.isRequired
    }),

    /**
     * The origin of the label relative to the top-left corner of the icon image, if a label is supplied by the marker.
     *
     * By default, the origin is located in the center point of the image.
     */
    labelOrigin: _propTypes2.default.shape({
      /**
       * The X coordinate.
       */
      x: _propTypes2.default.number.isRequired,
      /**
       * The Y coordinate.
       */
      y: _propTypes2.default.number.isRequired
    }),

    /**
     * The position of the image within a sprite, if any.
     */
    origin: _propTypes2.default.shape({
      /**
       * The X coordinate.
       */
      x: _propTypes2.default.number.isRequired,
      /**
       * The Y coordinate.
       */
      y: _propTypes2.default.number.isRequired
    }),

    /**
     * The display size of the sprite or image.
     *
     * When using sprites, you must specify the sprite size.
     *
     * If the size is not provided, it will be set when the image loads.
     */
    size: _propTypes2.default.shape({
      /**
       * The height along the y-axis, in pixels.
       */
      width: _propTypes2.default.number.isRequired,
      /**
       * The width along the x-axis, in pixels.
       */
      height: _propTypes2.default.number.isRequired
    }),
    /**
     * The size of the entire image after scaling, if any.
     *
     * Use this property to stretch/shrink an image or a sprite.
     */
    scaledSize: _propTypes2.default.shape({
      /**
       * The height along the y-axis, in pixels.
       */
      width: _propTypes2.default.number.isRequired,
      /**
       * The width along the x-axis, in pixels.
       */
      height: _propTypes2.default.number.isRequired
    })
  };
}