"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MarkerSymbol = undefined;

var _isEqual2 = require("lodash/isEqual");

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _pick2 = require("lodash/pick");

var _pick3 = _interopRequireDefault(_pick2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _MarkerContext = require("./MarkerContext");

var _MapContext = require("../internal/MapContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var pickProps = function pickProps(x) {
  return (0, _pick3.default)(x, ["path", "rotation", "scale", "anchor", "labelOrigin", "fillColor", "fillOpacity", "strokeColor", "strokeOpacity", "strokeWeight"]);
};

/**
 * Defines icon options of `Marker` component.
 *
 * **Usage:**
 *
 * ```javascript
 * import React from "react";
 * import { GoogleMap, Marker, MarkerSymbol } from "react-google-map-components"
 *
 * export default function GoogleMapWrapper(props) {
 *   return (
 *     <GoogleMap {...props} maps={google.maps}>
 *       <Marker
 *         position={props.center}
 *         icon={<MarkerSymbol path={props.markerPath} />}
 *       />
 *     </GoogleMap>
 *   );
 * }
 * ```
 *
 * **Google Maps Docs:**
 * * [google.maps.Symbol](https://developers.google.com/maps/documentation/javascript/reference#Symbol)
 */

var MarkerSymbol = exports.MarkerSymbol = function (_React$Component) {
  _inherits(MarkerSymbol, _React$Component);

  function MarkerSymbol() {
    _classCallCheck(this, MarkerSymbol);

    return _possibleConstructorReturn(this, (MarkerSymbol.__proto__ || Object.getPrototypeOf(MarkerSymbol)).apply(this, arguments));
  }

  _createClass(MarkerSymbol, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _context = this.context,
          mapContext = _context.mapContext,
          marker = _context.markerContext.marker;


      marker.setIcon(mapContext.createSymbol(this.props));
    }
  }, {
    key: "componentWillUpdate",
    value: function componentWillUpdate(nextProps) {
      var _context2 = this.context,
          mapContext = _context2.mapContext,
          markerContext = _context2.markerContext;


      var nextIcon = pickProps(nextProps);
      var prevIcon = pickProps(this.props);

      if (!(0, _isEqual3.default)(prevIcon, nextIcon)) {
        markerContext.marker.setIcon(mapContext.createSymbol(nextIcon));
      }
    }
  }, {
    key: "render",
    value: function render() {
      return null;
    }
  }]);

  return MarkerSymbol;
}(_react2.default.Component);

MarkerSymbol.contextTypes = {
  mapContext: _propTypes2.default.instanceOf(_MapContext.MapContext).isRequired,
  markerContext: _propTypes2.default.instanceOf(_MarkerContext.MarkerContext).isRequired
};

MarkerSymbol.defaultProps = {
  rotation: 0,
  scale: 1,
  anchor: { x: 0, y: 0 },
  labelOrigin: { x: 0, y: 0 },
  fillColor: "black",
  fillOpacity: 0,
  strokeColor: "black",
  strokeOpacity: 1,
  strokeWeight: 1
};

/* istanbul ignore else */
if (process.env.NODE_ENV !== "production") {
  MarkerSymbol.propTypes = {
    /**
     * Built-in symbol path, or a custom path expressed using
     * [SVG path notation](http://www.w3.org/TR/SVG/paths.html#PathData).
     */
    path: _propTypes2.default.string.isRequired,

    /**
     * The angle by which to rotate the symbol, expressed clockwise in degrees.
     */
    rotation: _propTypes2.default.number,

    /**
     * The amount by which the symbol is scaled in size.
     */
    scale: _propTypes2.default.number,

    /**
     * The position of the symbol relative to the marker.
     * The coordinates of the symbol's path are translated left and up by the
     * anchor's x and y coordinates respectively.
     */
    anchor: _propTypes2.default.shape({
      /**
       * The X coordinate.
       */
      x: _propTypes2.default.number.isRequired,
      /**
       * The Y coordinate.
       */
      y: _propTypes2.default.number.isRequired
    }),

    /**
     * The origin of the label relative to the origin of the path,
     * if label is supplied by the marker.
     */
    labelOrigin: _propTypes2.default.shape({
      /**
       * The X coordinate.
       */
      x: _propTypes2.default.number.isRequired,
      /**
       * The Y coordinate.
       */
      y: _propTypes2.default.number.isRequired
    }),

    /**
     * The symbol's fill color.
     *
     * All CSS3 colors are supported except for extended named colors.
     */
    fillColor: _propTypes2.default.string,

    /**
     * The symbol's fill opacity.
     */
    fillOpacity: _propTypes2.default.number,

    /**
     * The symbol's stroke color.
     *
     * All CSS3 colors are supported except for extended named colors.
     */
    strokeColor: _propTypes2.default.string,

    /**
     * The symbol's stroke opacity.
     */
    strokeOpacity: _propTypes2.default.number,

    /**
     * The symbol's stroke weight.
     */
    strokeWeight: _propTypes2.default.number
  };
}